;; ┌─┐┬─┐┌─┐┌─┐┬ ┬┬┌─┐┌─┐
;; │ ┬├┬┘├─┤├─┘├─┤││  └─┐
;; └─┘┴└─┴ ┴┴  ┴ ┴┴└─┘└─┘
;;       ┌─┐┌┐┌┌┬┐       
;;       ├─┤│││ ││       
;;       ┴ ┴┘└┘─┴┘       
;;     ┌┬┐┌─┐┌┬┐┌─┐      
;;      ││├─┤ │ ├─┤      
;;     ─┴┘┴ ┴ ┴ ┴ ┴      

;; load in the s-exp of graphics and pet data

(load "~/hobbies/uwu/version-1/uwu-graphics.el")

;; ┬┌─┌─┐┬ ┬┌┬┐┌─┐┌─┐
;; ├┴┐├┤ └┬┘│││├─┤├─┘
;; ┴ ┴└─┘ ┴ ┴ ┴┴ ┴┴  

(defvar uwu-mode-map nil "Keymap for `uwu-mode'.")

(progn
  (setq uwu-mode-map (make-sparse-keymap))
;;  (define-key uwu-mode-map (kbd "r") #'uwu--previous-pet)
;;  (define-key uwu-mode-map (kbd "s") #'uwu--next-pet)
  (define-key uwu-mode-map (kbd "t") #'uwu--print-pet-types)
  (define-key uwu-mode-map (kbd "w") #'uwu--window-setup)
  (define-key uwu-mode-map (kbd "h") #'uwu--help)
  (define-key uwu-mode-map (kbd "a") #'uwu--choose-alignment)
  (define-key uwu-mode-map (kbd "s") #'uwu--choose-skill)
  (define-key uwu-mode-map (kbd "g") #'uwu--choose-genus))
  

;; ┌┬┐┌─┐ ┬┌─┐┬─┐ 
;; │││├─┤ ││ │├┬┘ 
;; ┴ ┴┴ ┴└┘└─┘┴└─ 
;; ┌┬┐┌─┐┌┬┐┌─┐   
;; ││││ │ ││├┤    
;; ┴ ┴└─┘─┴┘└─┘   
;; ┌─┐┌─┐┌┬┐┬ ┬┌─┐
;; └─┐├┤  │ │ │├─┘
;; └─┘└─┘ ┴ └─┘┴  

(defconst uwu-version "1.0.0"
  "Current version of uwu.")

(defgroup uwu nil
  "A virtual pet simulator."
  :prefix "uwu-"
  :group 'games)

(defcustom uwu-mode-hook nil
  "Hook run when entering uwu mode."
  :type 'hook
  :group 'uwu)

(define-derived-mode uwu-mode fundamental-mode "uwu"
  "The major mode of the uwu virtual pet simulator"
  :group 'uwu)

;; ┌─┐┌─┐┌─┐┬┌─┌─┐┌─┐┌─┐
;; ├─┘├─┤│  ├┴┐├─┤│ ┬├┤ 
;; ┴  ┴ ┴└─┘┴ ┴┴ ┴└─┘└─┘

;; provide package

(provide 'uwu)

;; ┬  ┬┌─┐┬─┐┬┌─┐┌┐ ┬  ┌─┐┌─┐
;; └┐┌┘├─┤├┬┘│├─┤├┴┐��  ├┤ └─┐
;;  └┘ ┴ ┴┴└─┴┴ ┴└─┘┴─┘└─┘└─┘

(defvar *movement* 0)

(defvar *alignment-idle* 'lawful)
(defvar *skill-idle* 'smart)
(defvar *genus-idle* 'normie)

;; ┌─┐┬ ┬┌┐┌┌─┐┌┬┐┬┌─┐┌┐┌┌─┐
;; ├┤ │ │││││   │ ││ ││││└─┐
;; └  └─┘┘└┘└─┘ ┴ ┴└─┘┘└┘└─┘

(defun pet-movement (pet-gfx)

  (let ((rng-move (random 5)))

    (cond ((and
	    (= 3 rng-move)
	    (> *movement* 0))

	   (decf *movement*))

	  (
	   (and
	    (= 4 rng-move)
	    (< *movement* 9))

	   (incf *movement*)))

    (princ "╔═════════════════════╗" (get-buffer "*uwu-visualiser*"))
    (terpri (get-buffer "*uwu-visualiser*"))
    
    (princ
     (concat
      (substring "         " *movement*)

      (nth
       rng-move
       (cdr pet-gfx))) ; here we use the CDR of the gfx key-value
		       ; field, which gives us the graphics, and
		       ; eliminates the 'key' element.
     
     (get-buffer "*uwu-visualiser*"))
    (terpri (get-buffer "*uwu-visualiser*"))
        

    (princ "╚═════════════════════╝" (get-buffer "*uwu-visualiser*"))))

(defun blank-and-draw-frame (pet-gfx)
  (set-buffer "*uwu-visualiser*")
  (erase-buffer)
  (pet-movement pet-gfx))

(defun adult-pet-gfx-lookup (alignment skill genus)
      
    (assoc 'gfx
	   (assoc genus
		  (assoc skill
			 (assoc alignment +adult-data+)))))

(defun switch-pet-gfx (alignment skill genus)

  (pet-movement (adult-pet-gfx-lookup alignment skill genus)))

(defun one-frame-of-animation (alignment skill genus)

  (blank-and-draw-frame (adult-pet-gfx-lookup alignment skill genus)))


(defun uwu--window-setup ()
  (interactive)
  (display-buffer
   (get-buffer-create "*uwu-visualiser*")
                  '(display-buffer-same-window . (())))

  (display-buffer
   (get-buffer-create "*human-pet-interaction*")
		  '(display-buffer-below-selected . (()))))

(defun uwu-visualiser ()
  (loop
   (one-frame-of-animation *alignment-idle* *skill-idle* *genus-idle*)

   (sleep-for 0.5)))

(defun uwu--print-pet-types ()
  (interactive)
  (set-buffer "*human-pet-interaction*")
  (erase-buffer)
  (princ "| (Pick one) | (Pick one) | (Pick one) |" (get-buffer "*human-pet-interaction*"))
  (terpri (get-buffer "*human-pet-interaction*"))
  (princ "| Alignment  | Skill      | Genus      |" (get-buffer "*human-pet-interaction*"))
  (terpri (get-buffer "*human-pet-interaction*"))
  (princ "|:----------:|:----------:|:----------:|" (get-buffer "*human-pet-interaction*"))
  (terpri (get-buffer "*human-pet-interaction*"))
  (princ "| Lawful     | Smart      | Normie     |" (get-buffer "*human-pet-interaction*"))
  (terpri (get-buffer "*human-pet-interaction*"))
  (princ "| Chaotic    | Creative   | Active     |" (get-buffer "*human-pet-interaction*"))
  (terpri (get-buffer "*human-pet-interaction*"))
  (princ "|            | Social     | Funny      |" (get-buffer "*human-pet-interaction*"))
  (terpri (get-buffer "*human-pet-interaction*"))
  (princ "|            |            | Loud       |" (get-buffer "*human-pet-interaction*")))

(defun uwu--help ()
  (interactive)
  (set-buffer "*human-pet-interaction*")
  (erase-buffer)
  (princ "" (get-buffer "*human-pet-interaction*"))
  (terpri (get-buffer "*human-pet-interaction*"))
  (terpri (get-buffer "*human-pet-interaction*"))
  (princ "~ uwu.el help dialogue ~" (get-buffer "*human-pet-interaction*"))
  (terpri (get-buffer "*human-pet-interaction*"))
  (terpri (get-buffer "*human-pet-interaction*"))
  (princ "keyboard letter:  ||  action:" (get-buffer "*human-pet-interaction*"))
  (terpri (get-buffer "*human-pet-interaction*"))
  (princ "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~" (get-buffer "*human-pet-interaction*"))
  (terpri (get-buffer "*human-pet-interaction*"))
  (princ "       h          ||   help dialogue" (get-buffer "*human-pet-interaction*"))
  (terpri (get-buffer "*human-pet-interaction*"))
  (princ "       t          ||   table of pet types" (get-buffer "*human-pet-interaction*"))
  (terpri (get-buffer "*human-pet-interaction*"))
  (princ "       a          ||   choose pet alignment" (get-buffer "*human-pet-interaction*"))
  (terpri (get-buffer "*human-pet-interaction*"))
  (princ "       s          ||   choose pet skill" (get-buffer "*human-pet-interaction*"))
  (terpri (get-buffer "*human-pet-interaction*"))
  (princ "       g          ||   choose pet genus" (get-buffer "*human-pet-interaction*")))
;;; ╔═════════════════════╗
;;; <@(･ω･o< )@     
;;; ╚═════════════════════╝